@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2024 Vaclav Slavik
+ *  Copyright (C) 1999-2025 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,7 @@
 #include "catalog_po.h"
 #include "catalog_xliff.h"
 #include "catalog_json.h"
+#include "catalog_xcloc.h"
 
 #include "configuration.h"
 #include "errors.h"
@@ -589,6 +590,8 @@ wxString MaskForType(Catalog::Type t)
             return MaskForType("*.pot", _("POT Translation Templates"));
         case Catalog::Type::XLIFF:
             return MaskForType("*.xlf;*.xliff", _("XLIFF Translation Files"));
+        case Catalog::Type::XCLOC:
+            return MaskForType("*.xcloc", _("Xcode Localization Catalog"));
         case Catalog::Type::JSON:
             return MaskForType("*.json", _("JSON Translation Files"));
         case Catalog::Type::JSON_FLUTTER:
@@ -602,9 +605,9 @@ wxString MaskForType(Catalog::Type t)
 
 wxString Catalog::GetAllTypesFileMask()
 {
-    return MaskForType("*.po;*.pot;*.xlf;*.xliff;*.json;*.arb", _("All Translation Files"), /*showExt=*/false) +
+    return MaskForType("*.po;*.pot;*.xlf;*.xliff;*.xcloc;*.json;*.arb", _("All Translation Files"), /*showExt=*/false) +
         "|" +
-        GetTypesFileMask({ Type::PO, Type::POT, Type::XLIFF, Type::JSON, Type::JSON_FLUTTER });
+        GetTypesFileMask({ Type::PO, Type::POT, Type::XLIFF, Type::JSON, Type::JSON_FLUTTER, Type::XCLOC });
 }
 
 wxString Catalog::GetTypesFileMask(std::initializer_list<Type> types)
@@ -881,6 +884,9 @@ std::string CatalogItem::GetFormatFlag() const
 
 void CatalogItem::SetFuzzy(bool fuzzy)
 {
+    if (fuzzy == m_isFuzzy)
+        return;
+
     if (!fuzzy && m_isFuzzy)
         m_oldMsgid.clear();
     m_isFuzzy = fuzzy;
@@ -888,6 +894,16 @@ void CatalogItem::SetFuzzy(bool fuzzy)
     UpdateInternalRepresentation();
 }
 
+void CatalogItem::SetComment(const wxString& c)
+{
+    if (c == m_comment)
+        return;
+
+    m_comment = c;
+    UpdateInternalRepresentation();
+}
+
+
 wxString CatalogItem::GetTranslation(unsigned idx) const
 {
     if (idx >= GetNumberOfTranslations())
@@ -968,17 +984,21 @@ void CatalogItem::SetTranslationFromSource()
 
 void CatalogItem::ClearTranslation()
 {
+    bool modified = m_isFuzzy != false;
     m_isFuzzy = false;
     m_isPreTranslated = false;
     m_isTranslated = false;
     for (auto& t: m_translations)
     {
         if (!t.empty())
-            m_isModified = true;
+            modified = true;
         t.clear();
     }
 
-    UpdateInternalRepresentation();
+    m_isModified = modified;
+
+    if (modified)
+        UpdateInternalRepresentation();
 }
 
 unsigned CatalogItem::GetPluralFormsCount() const
@@ -1110,6 +1130,7 @@ CatalogPtr Catalog::Create(Type type)
             return CatalogPtr(new POCatalog(type));
 
         case Type::XLIFF:
+        case Type::XCLOC:
         case Type::JSON:
         case Type::JSON_FLUTTER:
             wxFAIL_MSG("empty XLIFF/JSON creation not implemented");
@@ -1139,6 +1160,10 @@ CatalogPtr Catalog::Create(const wxString& filename, int flags)
     {
         cat = JSONCatalog::Open(filename);
     }
+    else if (XCLOCCatalog::CanLoadFile(ext))
+    {
+        cat = XCLOCCatalog::Open(filename);
+    }
 
     if (!cat)
         throw Exception(_("The file is in a format not recognized by Poedit."));
