@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2024 Vaclav Slavik
+ *  Copyright (C) 1999-2025 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -219,7 +219,7 @@ class CatalogItem
         void SetPreTranslated(bool pre) { m_isPreTranslated = pre; }
 
         /// Sets the comment.
-        void SetComment(const wxString& c) { m_comment = c; }
+        void SetComment(const wxString& c);
 
 
         // -------------------------------------------------------------------
@@ -338,6 +338,7 @@ class Catalog
             PO,
             POT,
             XLIFF,
+            XCLOC,
             JSON,
             JSON_FLUTTER
         };
