@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2024 Vaclav Slavik
+ *  Copyright (C) 1999-2025 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -42,6 +42,7 @@
 #include <wx/datetime.h>
 #include <wx/config.h>
 #include <wx/textfile.h>
+#include <wx/scopeguard.h>
 #include <wx/stdpaths.h>
 #include <wx/strconv.h>
 #include <wx/memtext.h>
@@ -50,10 +51,6 @@
 #include <set>
 #include <algorithm>
 
-#ifdef __WXOSX__
-#import <Foundation/Foundation.h>
-#endif
-
 // TODO: split into different file
 #if wxUSE_GUI
     #include <wx/msgdlg.h>
@@ -436,6 +433,9 @@ bool POCatalogParser::Parse()
                 mtranslations.Add(str);
             }
 
+            if (m_ignoreTranslations)
+                mtranslations.clear();
+
             if (!OnEntry(mstr, msgid_plural, true,
                          has_context, msgctxt,
                          mtranslations,
@@ -471,10 +471,14 @@ bool POCatalogParser::Parse()
 
                 deletedLines.Add(line);
             }
-            if (!OnDeletedEntry(deletedLines,
-                                mflags, mrefs, mcomment, mextractedcomments, mlinenum))
+
+            if (!m_ignoreTranslations)
             {
-                return false;
+                if (!OnDeletedEntry(deletedLines,
+                                    mflags, mrefs, mcomment, mextractedcomments, mlinenum))
+                {
+                    return false;
+                }
             }
 
             mcomment = mstr = msgid_plural = mflags = wxEmptyString;
@@ -1128,22 +1132,6 @@ wxString FormatStringForFile(const wxString& text)
 } // anonymous namespace
 
 
-#ifdef __WXOSX__
-
-@interface CompiledMOFilePresenter : NSObject<NSFilePresenter>
-@property (atomic, copy) NSURL *presentedItemURL;
-@property (atomic, copy) NSURL *primaryPresentedItemURL;
-@end
-
-@implementation CompiledMOFilePresenter
-- (NSOperationQueue *)presentedItemOperationQueue {
-     return [NSOperationQueue mainQueue];
-}
-@end
-
-#endif // __WXOSX__
-
-
 bool POCatalog::Save(const wxString& po_file, bool save_mo,
                      ValidationResults& validation_results, CompilationStatus& mo_compilation_status)
 {
@@ -1206,13 +1194,14 @@ bool POCatalog::Save(const wxString& po_file, bool save_mo,
     // to the usual format. This is a (barely) passable fix for #25 until
     // proper preservation of formatting is implemented.
 
-    int msgcat_ok = false;
+    bool msgcat_ok = false;
     {
+        wxArrayString args { "msgcat", "--force-po" };
+
         int wrapping = DEFAULT_WRAPPING;
         if (wxConfig::Get()->ReadBool("keep_crlf", true))
             wrapping = m_fileWrappingWidth;
 
-        wxString wrappingFlag;
         if (wrapping == DEFAULT_WRAPPING)
         {
             if (wxConfig::Get()->ReadBool("wrap_po_files", true))
@@ -1226,26 +1215,26 @@ bool POCatalog::Save(const wxString& po_file, bool save_mo,
         }
 
         if (wrapping == NO_WRAPPING)
-            wrappingFlag = " --no-wrap";
+            args.push_back("--no-wrap");
         else if (wrapping != DEFAULT_WRAPPING)
-            wrappingFlag.Printf(" --width=%d", wrapping);
+            args.push_back(wxString::Format("--width=%d", wrapping));
 
         TempOutputFileFor po_file_temp2_obj(po_file_temp);
         const wxString po_file_temp2 = po_file_temp2_obj.FileName();
-        auto msgcatCmd = wxString::Format("msgcat --force-po%s -o %s %s",
-                                          wrappingFlag,
-                                          QuoteCmdlineArg(po_file_temp2),
-                                          QuoteCmdlineArg(po_file_temp));
-        wxLogTrace("poedit", "formatting file with %s", msgcatCmd);
+
+        args.push_back("-o");
+        args.push_back(po_file_temp2);
+        args.push_back(po_file_temp);
+
+        wxLogTrace("poedit", "formatting file with %s", wxJoin(args, ' '));
 
         // Ignore msgcat errors output (but not exit code), because it
         //   a) complains about things DoValidate() already complained above
         //   b) issues warnings about source-extraction things (e.g. using non-ASCII
         //      msgids) that, while correct, are not something a *translator* can
         //      do anything about.
-        wxLogNull null;
-        msgcat_ok = ExecuteGettext(msgcatCmd) &&
-                    wxFileExists(po_file_temp2);
+        GettextRunner runner;
+        msgcat_ok = runner.run_sync(args) && wxFileExists(po_file_temp2);
 
         // msgcat always outputs Unix line endings, so we need to reformat the file
         if (msgcat_ok && outputCrlf == wxTextFileType_Dos)
@@ -1298,14 +1287,7 @@ bool POCatalog::Save(const wxString& po_file, bool save_mo,
         {
             // Ignore msgfmt errors output (but not exit code), because it
             // complains about things DoValidate() already complained above.
-            wxLogNull null;
-
-            if ( ExecuteGettext
-                  (
-                      wxString::Format("msgfmt -o %s %s",
-                                       QuoteCmdlineArg(mo_file_temp),
-                                       QuoteCmdlineArg(CliSafeFileName(po_file)))
-                  ) )
+            if (GettextRunner().run_sync("msgfmt", "-o", mo_file_temp, CliSafeFileName(po_file)))
             {
                 mo_compilation_status = CompilationStatus::Success;
             }
@@ -1329,45 +1311,11 @@ bool POCatalog::Save(const wxString& po_file, bool save_mo,
         // Move the MO from temporary location to the final one, if it was created
         if (mo_compilation_status == CompilationStatus::Success)
         {
-#ifdef __WXOSX__
-            NSURL *mofileUrl = [NSURL fileURLWithPath:str::to_NS(mo_file)];
-            NSURL *mofiletempUrl = [NSURL fileURLWithPath:str::to_NS(mo_file_temp)];
-            bool sandboxed = (getenv("APP_SANDBOX_CONTAINER_ID") != NULL);
-            CompiledMOFilePresenter *presenter = nil;
-            if (sandboxed)
-            {
-                presenter = [CompiledMOFilePresenter new];
-                presenter.presentedItemURL = mofileUrl;
-                presenter.primaryPresentedItemURL = [NSURL fileURLWithPath:str::to_NS(po_file)];
-                [NSFileCoordinator addFilePresenter:presenter];
-                [NSFileCoordinator filePresenters];
-            }
-            NSFileCoordinator *coo = [[NSFileCoordinator alloc] initWithFilePresenter:presenter];
-            [coo coordinateWritingItemAtURL:mofileUrl options:NSFileCoordinatorWritingForReplacing error:nil byAccessor:^(NSURL *newURL) {
-                NSURL *resultingUrl;
-                BOOL ok = [[NSFileManager defaultManager] replaceItemAtURL:newURL
-                                                             withItemAtURL:mofiletempUrl
-                                                            backupItemName:nil
-                                                                   options:0
-                                                          resultingItemURL:&resultingUrl
-                                                                     error:nil];
-                if (!ok)
-                {
-                    wxLogError(_(L"Couldn’t save file %s."), mo_file.c_str());
-                    mo_compilation_status = CompilationStatus::Error;
-                }
-            }];
-            if (sandboxed)
-            {
-                [NSFileCoordinator removeFilePresenter:presenter];
-            }
-#else // !__WXOSX__
             if ( !mo_file_temp_obj.Commit() )
             {
                 wxLogError(_(L"Couldn’t save file %s."), mo_file.c_str());
                 mo_compilation_status = CompilationStatus::Error;
             }
-#endif // __WXOSX__/!__WXOSX__
         }
     }
 
@@ -1415,7 +1363,9 @@ bool POCatalog::CompileToMO(const wxString& mo_file,
     TempDirectory tmpdir;
     if ( !tmpdir.IsOk() )
         return false;
-    wxString po_file_temp = tmpdir.CreateFileName("output.po");
+
+    wxFileName mofn(mo_file);
+    wxString po_file_temp = tmpdir.CreateFileName(mofn.GetName() + ".po");
 
     if ( !DoSaveOnly(po_file_temp, wxTextFileType_Unix) )
     {
@@ -1428,16 +1378,13 @@ bool POCatalog::CompileToMO(const wxString& mo_file,
     TempOutputFileFor mo_file_temp_obj(mo_file);
     const wxString mo_file_temp = mo_file_temp_obj.FileName();
 
-    {
-        // Ignore msgfmt errors output (but not exit code), because it
-        // complains about things DoValidate() already complained above.
-        wxLogNull null;
-        ExecuteGettext(wxString::Format("msgfmt -o %s %s",
-                                        QuoteCmdlineArg(mo_file_temp),
-                                        QuoteCmdlineArg(po_file_temp)));
-    }
+    GettextRunner runner;
+    runner.run_sync("msgfmt", "-o", mo_file_temp, po_file_temp);
 
-    // Don't check return code:
+    // Ignore msgfmt errors output (but not exit code), because it
+    // complains about things DoValidate() already complained above.
+    //
+    // Don't check return code either:
     // msgfmt has the ugly habit of sometimes returning non-zero
     // exit code, reporting "fatal errors" and *still* producing a usable
     // .mo file. If this happens, don't pretend the file wasn't created.
@@ -1651,9 +1598,7 @@ bool POCatalog::FixDuplicateItems()
         return false;
     }
 
-    ExecuteGettext(wxString::Format("msguniq -o %s %s",
-                                    QuoteCmdlineArg(CliSafeFileName(po_file_fixed)),
-                                    QuoteCmdlineArg(CliSafeFileName(po_file_temp))));
+    GettextRunner().run_sync("msguniq", "-o", CliSafeFileName(po_file_fixed), CliSafeFileName(po_file_temp));
 
     if (!wxFileName::FileExists(po_file_fixed))
         return false;
@@ -1695,28 +1640,23 @@ Catalog::ValidationResults POCatalog::Validate(const wxString& fileWithSameConte
 
 void POCatalog::ValidateWithMsgfmt(Catalog::ValidationResults& res, const wxString& po_file)
 {
-    GettextErrors err;
-    ExecuteGettextAndParseOutput
-    (
-        wxString::Format("msgfmt -o /dev/null -c %s", QuoteCmdlineArg(CliSafeFileName(po_file))),
-        err
-    );
+    GettextRunner gtr;
+    auto output = gtr.run_sync("msgfmt", "-o", "/dev/null", "-c", CliSafeFileName(po_file));
+    auto errors = gtr.parse_stderr(output);
 
-    res.errors += (int)err.size();
-
-    for ( GettextErrors::const_iterator i = err.begin(); i != err.end(); ++i )
+    for (auto& i: errors.items)
     {
-        if ( i->line != -1 )
+        if (i.has_location())
         {
-            auto item = FindItemByLine(i->line);
-            if ( item )
+            auto item = FindItemByLine(i.line);
+            if (item)
             {
-                item->SetIssue(CatalogItem::Issue::Error, i->text);
-                continue;
+                res.errors++;
+                item->SetIssue(CatalogItem::Issue::Error, i.text);
             }
         }
-        // if not matched to an item:
-        wxLogError(i->text);
+        // else: ignore msgfmt output w/o a location because msgfmt outputs status information
+        //       (e.g. "N errors found") to stderr too
     }
 }
 
@@ -1777,6 +1717,7 @@ POCatalogPtr POCatalog::CreateFromPOT(POCatalogPtr pot)
 bool POCatalog::Merge(const POCatalogPtr& refcat)
 {
     wxString oldname = m_fileName;
+    wxON_BLOCK_EXIT_SET(m_fileName, oldname);
 
     TempDirectory tmpdir;
     if ( !tmpdir.IsOk() )
@@ -1789,25 +1730,23 @@ bool POCatalog::Merge(const POCatalogPtr& refcat)
     refcat->DoSaveOnly(tmp1, wxTextFileType_Unix);
     DoSaveOnly(tmp2, wxTextFileType_Unix);
 
-    wxString flags("-q --force-po --previous");
+    std::vector<wxString> args { "msgmerge", "-q", "--force-po", "--previous" };
     if (Config::MergeBehavior() == Merge_None)
     {
-        flags += " --no-fuzzy-matching";
+        args.push_back(" --no-fuzzy-matching");
     }
 
-    bool succ = ExecuteGettext
-                (
-                    wxString::Format
-                    (
-                        "msgmerge %s -o %s %s %s",
-                        flags,
-                        QuoteCmdlineArg(tmp3),
-                        QuoteCmdlineArg(tmp2),
-                        QuoteCmdlineArg(tmp1)
-                    )
-                );
+    args.push_back("-o");
+    args.push_back(tmp3);
+    args.push_back(tmp2);
+    args.push_back(tmp1);
 
-    if (succ)
+    GettextRunner runner;
+    auto output = runner.run_sync(args);
+    // FIXME: Don't do that here, report as part of return value instead
+    runner.parse_stderr(output).log_errors();
+
+    if (output)
     {
         const wxString charset = m_header.Charset;
 
@@ -1821,7 +1760,5 @@ bool POCatalog::Merge(const POCatalogPtr& refcat)
         m_header.Charset = charset;
     }
 
-    m_fileName = oldname;
-
-    return succ;
+    return (bool)output;
 }
