@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2018-2024 Vaclav Slavik
+ *  Copyright (C) 2018-2025 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -212,6 +212,31 @@ inline bool is_numeric_only(const std::string& s)
 }
 
 
+inline void xliff_prefix_id(std::string& id, xml_node node, const char *nameAttr)
+{
+    std::string prefix;
+    for (auto p = node.parent(); p; p = p.parent())
+    {
+        if (strcmp(p.name(), "group") == 0)
+        {
+            auto gid = p.attribute(nameAttr);
+            if (!gid)
+                gid = p.attribute("id");
+            if (gid)
+            {
+                prefix.insert(0, " / ");
+                prefix.insert(0, gid.value());
+                continue; // go to grandparent
+            }
+        }
+        // stop at first non-group or unnamed parent:
+        break;
+    }
+
+    if (!prefix.empty())
+        id.insert(0, prefix);
+}
+
 
 class MetadataExtractor : public pugi::xml_tree_walker
 {
@@ -478,26 +503,18 @@ bool XLIFFCatalog::CanLoadFile(const wxString& extension)
 }
 
 
-std::shared_ptr<XLIFFCatalog> XLIFFCatalog::Open(const wxString& filename)
+std::shared_ptr<XLIFFCatalog> XLIFFCatalog::OpenImpl(const wxString& filename, InstanceCreatorImpl& creator)
 {
     xml_document doc;
     auto result = doc.load_file(filename.fn_str(), PUGI_PARSE_FLAGS);
     if (!result)
         throw XLIFFReadException(result.description());
 
-    std::shared_ptr<XLIFFCatalog> cat;
-
     auto xliff_root = doc.child("xliff");
     std::string xliff_version = xliff_root.attribute("version").value();
-    if (xliff_version == "1.0")
-        cat.reset(new XLIFF1Catalog(std::move(doc), 0));
-    else if (xliff_version == "1.1")
-        cat.reset(new XLIFF1Catalog(std::move(doc), 1));
-    else if (xliff_version == "1.2")
-        cat.reset(new XLIFF1Catalog(std::move(doc), 2));
-    else if (xliff_version == "2.0" || xliff_version == "2.1")
-        cat.reset(new XLIFF2Catalog(std::move(doc)));
-    else
+
+    auto cat = creator.CreateFromDoc(std::move(doc), xliff_version);
+    if (!cat)
         throw XLIFFReadException(wxString::Format(_("unsupported version (%s)"), xliff_version));
 
     cat->Parse(xliff_root);
@@ -506,6 +523,30 @@ std::shared_ptr<XLIFFCatalog> XLIFFCatalog::Open(const wxString& filename)
 }
 
 
+std::shared_ptr<XLIFFCatalog> XLIFFCatalog::Open(const wxString& filename)
+{
+    struct Creator : public InstanceCreatorImpl
+    {
+        std::shared_ptr<XLIFFCatalog> CreateFromDoc(pugi::xml_document&& doc, const std::string& xliff_version) override
+        {
+            if (xliff_version == "1.0")
+                return std::make_shared<XLIFF1Catalog>(std::move(doc), 0);
+            else if (xliff_version == "1.1")
+                return std::make_shared<XLIFF1Catalog>(std::move(doc), 1);
+            else if (xliff_version == "1.2")
+                return std::make_shared<XLIFF1Catalog>(std::move(doc), 2);
+            else if (xliff_version == "2.0" || xliff_version == "2.1")
+                return std::make_shared<XLIFF2Catalog>(std::move(doc));
+            else
+                return nullptr;
+        }
+    };
+
+    Creator c;
+    return OpenImpl(filename, c);
+}
+
+
 bool XLIFFCatalog::Save(const wxString& filename, bool /*save_mo*/,
                         ValidationResults& validation_results,
                         CompilationStatus& /*mo_compilation_status*/)
@@ -575,7 +616,10 @@ public:
             id = node.attribute("id").value();
         // some tools (e.g. Xcode, tool-id="com.apple.dt.xcode") use ID same as text; numeric only is useless to translator too
         if (!id.empty() && id != m_string && !is_numeric_only(id))
+        {
+            xliff_prefix_id(id, node, "resname");
             m_symbolicId = str::to_wx(id);
+        }
 
         auto target = node.child("target");
         if (target)
@@ -586,7 +630,7 @@ public:
             std::string state = target.attribute("state").value();
             if (state == "needs-adaptation" || state == "needs-l10n")
                 m_isFuzzy = true;
-            else if (m_isTranslated && (state == "new" || state == "needs-translation"))
+            else if (m_isTranslated && (state == "new" || state == "needs-translation" || state == "needs-review-translation"))
                 m_isFuzzy = true;
         }
         else
@@ -758,7 +802,10 @@ public:
             id = unit().attribute("id").value();
         // some tools (e.g. Xcode, tool-id="com.apple.dt.xcode") use ID same as text; numeric only is useless to translator too
         if (!id.empty() && id != m_string && !is_numeric_only(id))
+        {
+            xliff_prefix_id(id, unit(), "name");
             m_symbolicId = str::to_wx(id);
+        }
 
         auto target = node.child("target");
         if (target)
