@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2018-2024 Vaclav Slavik
+ *  Copyright (C) 2018-2025 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -126,6 +126,14 @@ protected:
     XLIFFCatalog(pugi::xml_document&& doc)
         : Catalog(Type::XLIFF), m_doc(std::move(doc)) {}
 
+    struct InstanceCreatorImpl
+    {
+        virtual ~InstanceCreatorImpl() {}
+        virtual std::shared_ptr<XLIFFCatalog> CreateFromDoc(pugi::xml_document&& doc, const std::string& xliff_version) = 0;
+    };
+
+    static std::shared_ptr<XLIFFCatalog> OpenImpl(const wxString& filename, InstanceCreatorImpl& creator);
+
     virtual void Parse(pugi::xml_node root) = 0;
 
 protected:
