@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2017-2024 Vaclav Slavik
+ *  Copyright (C) 2017-2025 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -97,10 +97,12 @@ const char * const GETTEXT_EXTENSIONS[] = {
 class GettextExtractorBase : public Extractor
 {
 public:
-    wxString Extract(TempDirectory& tmpdir,
-                     const SourceCodeSpec& sourceSpec,
-                     const std::vector<wxString>& files) const override
+    ExtractionOutput Extract(TempDirectory& tmpdir,
+                             const SourceCodeSpec& sourceSpec,
+                             const std::vector<wxString>& files) const override
     {
+        using subprocess::quote_arg;
+
         auto basepath = sourceSpec.BasePath;
 #ifdef __WXMSW__
         basepath = CliSafeFileName(basepath);
@@ -130,10 +132,10 @@ public:
         cmdline.Printf
         (
             "xgettext --force-po -o %s --directory=%s --files-from=%s --from-code=%s",
-            QuoteCmdlineArg(outfile),
-            QuoteCmdlineArg(basepath),
-            QuoteCmdlineArg(filelist.GetName()),
-            QuoteCmdlineArg(!sourceSpec.Charset.empty() ? sourceSpec.Charset : "UTF-8")
+            quote_arg(outfile),
+            quote_arg(basepath),
+            quote_arg(filelist.GetName()),
+            quote_arg(!sourceSpec.Charset.empty() ? sourceSpec.Charset : "UTF-8")
         );
 
         auto additional = GetAdditionalFlags();
@@ -142,7 +144,7 @@ public:
 
         for (auto& kw: sourceSpec.Keywords)
         {
-            cmdline += wxString::Format(" -k%s", QuoteCmdlineArg(kw));
+            cmdline += wxString::Format(" -k%s", quote_arg(kw));
         }
 
         wxString extraFlags;
@@ -158,10 +160,17 @@ public:
         if (!extraFlags.empty())
             cmdline += " " + extraFlags;
 
-        if (!ExecuteGettext(cmdline))
-            throw ExtractionException(ExtractionError::Unspecified);
+        GettextRunner runner;
+        auto output = runner.run_command_sync(cmdline);
+        auto err = runner.parse_stderr(output);
 
-        return outfile;
+        if (output.failed())
+        {
+            err.log_all();
+            throw ExtractionException(ExtractionError::Unspecified);
+        }
+
+        return {outfile, err};
     }
     
 protected:
