@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2000-2024 Vaclav Slavik
+ *  Copyright (C) 2000-2025 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -66,6 +66,7 @@
 #include "uilang.h"
 #include "errors.h"
 #include "extractors/extractor_legacy.h"
+#include "progress_ui.h"
 #include "spellchecking.h"
 #include "str_helpers.h"
 #include "utility.h"
@@ -200,8 +201,6 @@ public:
 
         m_compileMo = new wxCheckBox(this, wxID_ANY, _("Automatically compile MO file when saving"));
         sizer->Add(m_compileMo);
-        m_showSummary = new wxCheckBox(this, wxID_ANY, _("Show summary after updating files"));
-        sizer->Add(m_showSummary, wxSizerFlags().PXBorder(wxTOP));
 
         sizer->AddSpacer(PX(10));
 
@@ -280,7 +279,6 @@ public:
         m_userName->SetValue(cfg.Read("translator_name", wxEmptyString));
         m_userEmail->SetValue(cfg.Read("translator_email", wxEmptyString));
         m_compileMo->SetValue(cfg.ReadBool("compile_mo", true));
-        m_showSummary->SetValue(cfg.ReadBool("show_summary", false));
         m_focusToText->SetValue(cfg.ReadBool("focus_to_text", false));
 
         if (IsSpellcheckingAvailable())
@@ -313,7 +311,6 @@ public:
         cfg.Write("translator_name", m_userName->GetValue());
         cfg.Write("translator_email", m_userEmail->GetValue());
         cfg.Write("compile_mo", m_compileMo->GetValue());
-        cfg.Write("show_summary", m_showSummary->GetValue());
         cfg.Write("focus_to_text", m_focusToText->GetValue());
 
         if (IsSpellcheckingAvailable())
@@ -341,7 +338,7 @@ public:
 
 private:
     wxTextCtrl *m_userName, *m_userEmail;
-    wxCheckBox *m_compileMo, *m_showSummary, *m_focusToText, *m_spellchecking;
+    wxCheckBox *m_compileMo, *m_focusToText, *m_spellchecking;
     wxCheckBox *m_useFontList, *m_useFontText;
     wxFontPickerCtrl *m_fontList, *m_fontText;
 #if NEED_CHOOSELANG_UI
@@ -543,35 +540,16 @@ private:
             wxArrayString paths;
             dlg->GetPaths(paths);
 
-            wxProgressDialog progress(_("Translation Memory"),
-                                      _(L"Importing translations…"),
-                                      (int)paths.size() * 2 + 1,
-                                      this,
-                                      wxPD_APP_MODAL|wxPD_AUTO_HIDE|wxPD_CAN_ABORT);
             auto tm = TranslationMemory::Get().GetWriter();
-            int step = 0;
-            for (size_t i = 0; i < paths.size(); i++)
+
+            DoImportIntoTM(paths, [=](const wxString& p)
             {
-                try
-                {
-                    auto cat = Catalog::Create(paths[i]);
-                    if (!progress.Update(++step))
-                        break;
-                    tm->Insert(cat);
-                    if (!progress.Update(++step))
-                        break;
-                }
-                catch (...)
-                {
-                    wxLogError(_(L"Error loading translation file “%s”."), paths[i]);
-                    progress.Update(++step);
-                    if (!progress.Update(++step))
-                        break;
-                }
-            }
-            progress.Pulse(_(L"Finalizing…"));
-            tm->Commit();
-            UpdateStats();
+                Progress subprogress(1);
+                auto cat = Catalog::Create(p);
+                tm->Insert(cat);
+                tm->Commit();
+                return cat->GetCount();
+            });
         }
     }
 
@@ -595,45 +573,60 @@ private:
 
             wxArrayString paths;
             dlg->GetPaths(paths);
+            DoImportIntoTM(paths, [=](const wxString& p)
+            {
+                std::ifstream f;
+                f.open(p.fn_str());
+                int count = TMX::ImportFromFile(f, TranslationMemory::Get());
+                f.close();
+                return count;
+            });
+        }
+    }
 
-            wxProgressDialog progress(_("Translation Memory"),
-                                      _(L"Importing translations…"),
-                                      (int)paths.size() + 1,
-                                      this,
-                                      wxPD_APP_MODAL|wxPD_AUTO_HIDE|wxPD_CAN_ABORT);
-            progress.Pulse();
+    template<typename T>
+    void DoImportIntoTM(const wxArrayString& paths, T&& doImportFile)
+    {
+        auto cancellation = std::make_shared<dispatch::cancellation_token>();
+        wxWindowPtr<ProgressWindow> progress(new ProgressWindow(this, _(L"Importing translations…"), cancellation));
+        progress->SetErrorMessage(_("Importing translation memory failed."));
+
+        progress->RunTaskModal([=]() -> BackgroundTaskResult
+        {
+            Progress progress(paths.size());
+
+            int count = 0;
             for (auto p: paths)
             {
+                if (cancellation->is_cancelled())
+                    break;
+
+                auto pname = wxFileName(p).GetFullName();
+                Progress subprogress(1);
+                subprogress.message(wxString::Format(_(L"Importing from “%s”…"), pname));
+
                 try
                 {
-                    std::ifstream f;
-                    f.open(p.fn_str());
-                    TMX::ImportFromFile(f, TranslationMemory::Get());
-                    f.close();
-
-                    if (!progress.Pulse())
-                        break;
+                    count += doImportFile(p);
                 }
                 catch (...)
                 {
-                    wxWindowPtr<wxMessageDialog> err(new wxMessageDialog
-                    (
-                            this,
-                            wxString::Format(_(L"Importing translation memory from “%s” failed."), p),
-                            _("Import error"),
-                            wxOK | wxICON_ERROR
-                        ));
-                    err->SetExtendedMessage(DescribeCurrentException());
-                    // FIXME: can't use ShowWindowModalThenDo, as would be better, because multiple
-                    //        errors may occur in this loop. See https://github.com/vslavik/poedit/issues/748
-                    if (paths.size() == 1)
-                        err->ShowWindowModalThenDo([err](int){});
-                    else
-                        err->ShowModal();
+                    wxLogError(("%s: %s"), pname, DescribeCurrentException());
                 }
             }
-            UpdateStats();
-        }
+
+            if (count == 0)
+                return {};
+
+            return wxString::Format
+                   (
+                       // TRANSLATORS: %s is a (formatted) number here
+                       wxPLURAL("%s translation was imported.", "%s translations were imported.", count),
+                       wxNumberFormatter::ToString((long)count)
+                   );
+        });
+
+        UpdateStats();
     }
 
     void OnExportTMX(wxCommandEvent&)
@@ -655,32 +648,21 @@ private:
                 return;
 
             auto p = dlg->GetPath();
-            wxProgressDialog progress(_("Translation Memory"),
-                                      _(L"Exporting translations…"),
-                                      1,
-                                      this,
-                                      wxPD_APP_MODAL|wxPD_AUTO_HIDE|wxPD_CAN_ABORT);
-            progress.Pulse();
 
-            try
+            wxWindowPtr<ProgressWindow> progress(new ProgressWindow(this, _(L"Exporting translations…")));
+            progress->SetErrorMessage(wxString::Format(_(L"Exporting translation memory to “%s” failed."), wxFileName(p).GetFullName()));
+            progress->RunTaskModal([=]()
             {
+                TempOutputFileFor tempfile(p);
+
                 std::ofstream f;
-                f.open(p.fn_str());
+                f.open(tempfile.FileName().fn_str());
                 TMX::ExportToFile(TranslationMemory::Get(), f);
                 f.close();
-            }
-            catch (...)
-            {
-                wxWindowPtr<wxMessageDialog> err(new wxMessageDialog
-                (
-                        this,
-                        wxString::Format(_(L"Exporting translation memory to “%s” failed."), p),
-                        _("Export error"),
-                        wxOK | wxICON_ERROR
-                    ));
-                err->SetExtendedMessage(DescribeCurrentException());
-                err->ShowWindowModalThenDo([err](int){});
-            }
+
+                if ( !tempfile.Commit() )
+                    throw Exception(wxString::Format(_(L"Couldn’t save file %s."), wxFileName(p).GetFullName()));
+            });
         }
     }
 
